import 'dart:async';
import 'dart:collection';

import 'package:miodi/core/consts.dart';

import '../error/errors.dart';
import 'component.dart';
import 'providers.dart';

class MiodiConfiguration {
  final Map<String, Map<Type, Provider>> _factoryConfiguration = HashMap();
  final Map<String, Map<Type, Provider>> _singletonConfiguration = HashMap();
  final Map<String, Map<Type, Provider>> _customConfiguration = HashMap();

  final Map<String, Map<Type, AsyncProvider>> _factoryAsyncConfiguration = HashMap();
  final Map<String, Map<Type, AsyncProvider>> _singletonAsyncConfiguration = HashMap();
  final Map<String, Map<Type, AsyncProvider>> _customAsyncConfiguration = HashMap();

  MiodiConfiguration(List<MiodiComponent> components) {
    addComponents(components);
  }

  T provide<T>({String name = defaultProviderName}) {
    return provideFromType<T>(T, name: name);
  }

  T provideFromType<T> (Type type, {String name = defaultProviderName}) {
    if (_singletonConfiguration[name]!= null && _singletonConfiguration[name].containsKey(type)){
      return _singletonConfiguration[name][type].provide();
    }
    if (_factoryConfiguration[name]!= null && _factoryConfiguration[name].containsKey(type)){
      return _factoryConfiguration[name][type].provide();
    }
    if (_customConfiguration[name]!= null && _customConfiguration.containsKey(type)){
      return _customConfiguration[name][type].provide();
    }
    throw NotFoundProviderException(type);
  }

  Future<T> provideAsyncFromType<T> (Type type, {String name = defaultProviderName}) {
    if (_singletonAsyncConfiguration[name]!= null && _singletonAsyncConfiguration[name].containsKey(type)){
      return _singletonAsyncConfiguration[name][type].provide().then((v) => v as T);
    }
    if (_factoryAsyncConfiguration[name]!= null && _factoryAsyncConfiguration[name].containsKey(type)){
      return _factoryAsyncConfiguration[name][type].provide().then((v) => v as T);
    }
    if (_customAsyncConfiguration[name]!= null && _customAsyncConfiguration.containsKey(type)){
      return _customAsyncConfiguration[name][type].provide().then((v) => v as T);
    }
    throw NotFoundProviderException(type);
  }

  Future<T> provideAsync<T>({String name = defaultProviderName}) async {
    return provideAsyncFromType<T>(T, name: name);
  }

  void addComponents(List<MiodiComponent> components) {
    components.forEach((component) {
      _factoryConfiguration.addAll(component.factoryConfiguration);
      _singletonConfiguration.addAll(component.singletonConfiguration);
      _customConfiguration.addAll(component.customConfiguration);

      _factoryAsyncConfiguration.addAll(component.factoryAsyncConfiguration);
      _singletonAsyncConfiguration.addAll(component.singletonAsyncConfiguration);
      _customAsyncConfiguration.addAll(component.customAsyncConfiguration);
    });
  }
}