import 'package:miodi/core/consts.dart';

class FactoryProvider<T> extends Provider<T> {
  FactoryProvider(Type T, Creator<T> creator, {String name})
      : super(T, creator, name: name);

  @override
  T provide() => creator();
}

class SingletonProvider<T> extends Provider<T> {
  T _instance;
  final initOnStart;
  SingletonProvider(Type T, Creator<T> creator,
      {String name, this.initOnStart = false})
      : super(T, creator, name: name) {
    if (initOnStart) init();
  }

  void init() {
    _instance = creator();
  }

  @override
  T provide() {
    if (_instance == null) _instance = creator();
    return _instance;
  }
}

class FactoryAsyncProvider<T> extends AsyncProvider<T> {
  FactoryAsyncProvider(Type T, AsyncCreator<T> creator, {String name})
      : super(T, creator, name: name);

  @override
  Future<T> provide() => creator();

}

class SingletonAsyncProvider<T> extends AsyncProvider<T> {
  T _instance;
  Future<T> _initFuture;
  final initOnStart;
  SingletonAsyncProvider(Type T, AsyncCreator<T> creator,
      {String name, this.initOnStart = false})
      : super(T, creator, name: name) {
    if (initOnStart) init();
  }

  void init() async {
    if (_initFuture == null)
      _initFuture = creator();
    _instance = await _initFuture;
  }

  @override
  Future<T> provide() async {
    if (_instance == null)
      await init();
    return _instance;
  }
}

abstract class AsyncProvider<T> {
  final Type _type;
  final String name;
  final AsyncCreator<T> _creator;
  AsyncProvider(this._type, this._creator, {this.name = defaultProviderName});
  Type get provideType => _type;
  AsyncCreator<T> get creator => _creator;

  Future<T> provide();
}

abstract class Provider<T> {
  final Type _type;
  final String name;
  final Creator<T> _creator;
  Provider(this._type, this._creator, {this.name = defaultProviderName});
  Type get provideType => _type;
  Creator<T> get creator => _creator;

  T provide();
}

typedef T Creator<T>();
typedef Future<T> AsyncCreator<T>();